package com.jeeplus.test.program.configuration.projectList.service;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.jeeplus.core.query.QueryWrapperGenerator;
import com.jeeplus.sys.service.dto.UserDTO;
import com.jeeplus.sys.utils.DictUtils;
import com.jeeplus.sys.utils.StringUtils;
import com.jeeplus.sys.utils.UserUtils;
import com.jeeplus.test.mould.service.SerialnumTplService;
import com.jeeplus.test.oss.domain.WorkAttachment;
import com.jeeplus.test.oss.mapper.OssServiceMapper;
import com.jeeplus.test.oss.service.OssService;
import com.jeeplus.test.program.configuration.projectList.domain.*;
import com.jeeplus.test.program.configuration.projectList.mapper.*;
import com.jeeplus.test.program.configuration.projectList.service.dto.*;
import com.jeeplus.test.program.configuration.projectList.service.mapstruct.ProgramArchiveWrapper;
import com.jeeplus.test.program.configuration.projectList.service.mapstruct.ProgramAuditAssessPeopleWrapper;
import com.jeeplus.test.program.configuration.projectList.service.mapstruct.ProgramAuditWrapper;
import com.jeeplus.test.workContract.domain.WorkContractInfo;
import com.jeeplus.test.workContract.mapper.WorkContractInfoMapper;
import com.jeeplus.test.workContract.service.WorkContractService;
import com.jeeplus.test.workContract.service.dto.WorkAttachmentDto;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;

@Service
public class ProjectListService {

    @Resource
    private ProjectListMapper projectListMapper;
    @Resource
    private ProjectLinkMapper projectLinkMapper;
    @Resource
    private ProjectMemberMapper projectMemberMapper;
    @Resource
    private WorkContractInfoMapper workContractInfoMapper;
    @Resource
    private OssServiceMapper ossServiceMapper;
    @Resource
    private OssService ossService;
    @Resource
    private SerialnumTplService serialnumTplService;
    @Resource
    private WorkContractService workContractService;
    @Resource
    private ProgramAuditMapper programAuditMapper;
    @Resource
    private ProgramReportNoMapper programReportNoMapper;
    @Resource
    private ProgramAuditAssessPeopleMapper programAuditAssessPeopleMapper;
    @Resource
    private ProgramArchiveMapper programArchiveMapper;

    public String save(ProjectListDto dto) throws Exception{
        if (StringUtils.isNotEmpty(dto.getId())) {
            return update(dto);
        }
        return add(dto);
    }

    public String add(ProjectListDto dto) throws Exception{
        ProgramProjectListInfo info = new ProgramProjectListInfo();
        //获取当前登录人信息
        UserDTO userDTO = UserUtils.getCurrentUserDTO();
        // 保存项目管理列表主表信息
        BeanUtils.copyProperties(dto, info);
        String id = UUID.randomUUID().toString().replace("-", "");
        //合同编号生成
        String no = serialnumTplService.genSerialNum(userDTO.getCompanyDTO().getId(), ProjectListDto.BIZ_CODE);
        //项目立项号生成
        String approvalNo = serialnumTplService.genSerialNum(userDTO.getCompanyDTO().getId(), ProjectListDto.BIY_CODE);
        info.setId(id);
        info.setNo(no);
        info.setApprovalNo(approvalNo);
        info.setCreateBy(userDTO.getId());
        info.setCreateDate(new Date());
        info.setUpdateBy(userDTO.getId());
        info.setUpdateDate(new Date());
        info.setDelFlag(0);
        projectListMapper.insert(info);
        // 保存项目直接对接人信息
        if (CollectionUtils.isNotEmpty(dto.getLinks())) {
            for (ProgramProjectListLink link : dto.getLinks()) {
                link.setId(UUID.randomUUID().toString().replace("-", ""));
                link.setInfoId(id);
                link.setCreateBy(userDTO.getId());
                link.setCreateDate(new Date());
                link.setUpdateBy(userDTO.getId());
                link.setUpdateDate(new Date());
                link.setDelFlag(0);
                projectLinkMapper.insert(link);
            }
        }
        // 保存项目管理列表成员信息
        if (CollectionUtils.isNotEmpty(dto.getMembers())) {
            for (ProgramProjectListMember member : dto.getMembers()) {
                member.setId(UUID.randomUUID().toString().replace("-", ""));
                member.setInfoId(id);
                member.setCreateBy(userDTO.getId());
                member.setCreateDate(new Date());
                member.setUpdateBy(userDTO.getId());
                member.setUpdateDate(new Date());
                member.setDelFlag(0);
                projectMemberMapper.insert(member);
            }
        }
        // 保存附件信息
        if (CollectionUtils.isNotEmpty(dto.getFiles())) {
            workContractService.saveFiles(dto.getFiles(), userDTO, id);
        }
        // 三级校审添加
        ProgramAudit programAudit1 = new ProgramAudit();
        programAudit1.setAuditLevel("1");
        programAudit1.setProgramId(info.getId());
        programAudit1.setStatus("0");
        programAuditMapper.insert(programAudit1);
        ProgramAudit programAudit2 = new ProgramAudit();
        programAudit2.setAuditLevel("2");
        programAudit2.setProgramId(info.getId());
        programAudit2.setStatus("0");
        programAuditMapper.insert(programAudit2);
        ProgramAudit programAudit3 = new ProgramAudit();
        programAudit3.setAuditLevel("3");
        programAudit3.setProgramId(info.getId());
        programAudit3.setStatus("0");
        programAuditMapper.insert(programAudit3);
        // 项目归档添加
        ProgramArchive programArchive = new ProgramArchive();
        programArchive.setProgramId(info.getId());
        programArchive.setStatus("0");
        programArchiveMapper.insert(programArchive);
        return id;
    }

    public String saveFormArchive(ProgramArchiveDto programArchiveDto) {
        ProgramArchive programArchive = ProgramArchiveWrapper.INSTANCE.toEntity(programArchiveDto);
        if (StringUtils.isNotBlank(programArchive.getId())){
            programArchiveMapper.updateById(programArchive);
        }else{
            programArchiveMapper.insert(programArchive);
        }
        if (ObjectUtil.isNotEmpty(programArchiveDto)) {
            ossService.saveOrUpdateFileListFlag(programArchiveDto.getReportFileList(),programArchive.getId(),"report");
            ossService.saveOrUpdateFileListFlag(programArchiveDto.getDetailFileList(),programArchive.getId(),"detail");
            ossService.saveOrUpdateFileListFlag(programArchiveDto.getExplainFileList(),programArchive.getId(),"explain");
            ossService.saveOrUpdateFileListFlag(programArchiveDto.getPapersFileList(),programArchive.getId(),"papers");
            ossService.saveOrUpdateFileListFlag(programArchiveDto.getOtherFileList(),programArchive.getId(),"other");
        }
        return programArchive.getId();
    }

    public String saveFormThree(ProgramAuditDto programAuditDto) {
        UserDTO userDTO = UserUtils.getCurrentUserDTO();
        ProgramAudit programAudit = ProgramAuditWrapper.INSTANCE.toEntity(programAuditDto);
        if (StringUtils.isNotBlank(programAudit.getId())){
            programAuditMapper.updateById(programAudit);
        }else{
            programAuditMapper.insert(programAudit);
        }
        if (ObjectUtil.isNotEmpty(programAuditDto)) {
            //附件
            ossService.saveOrUpdateFileList(programAuditDto.getWorkAttachmentDtoList(),programAudit.getId(),"program");
            //专业评估人员
            if (CollectionUtil.isNotEmpty(programAuditDto.getProgramAuditAssessPeopleDtoList())) {
                programAuditAssessPeopleMapper.delete(new QueryWrapper<ProgramAuditAssessPeople>().lambda().eq(ProgramAuditAssessPeople::getAuditId,programAudit.getId()));
                programAuditDto.getProgramAuditAssessPeopleDtoList().stream().forEach(item->{
                    ProgramAuditAssessPeople programAuditAssessPeople = ProgramAuditAssessPeopleWrapper.INSTANCE.toEntity(item);
                    programAuditAssessPeopleMapper.insert(programAuditAssessPeople);
                });
            }else{
                programAuditAssessPeopleMapper.delete(new QueryWrapper<ProgramAuditAssessPeople>().lambda().eq(ProgramAuditAssessPeople::getAuditId,programAudit.getId()));
            }
        }
        return programAudit.getId();
    }

    public String update(ProjectListDto dto) {
        //获取当前登录人信息
        UserDTO userDTO = UserUtils.getCurrentUserDTO();
        // 修改项目管理列表主表信息
        ProgramProjectListInfo info = new ProgramProjectListInfo();
        BeanUtils.copyProperties(dto, info);
        info.setUpdateBy(userDTO.getId());
        info.setUpdateDate(new Date());
        info.setDelFlag(0);
        projectListMapper.updateById(info);
        // 修改联系人信息(先删除原有数据，再保存)
        if (CollectionUtils.isNotEmpty(dto.getLinks())) {
            projectLinkMapper.deleteByInfoId(dto.getId());
            for (ProgramProjectListLink link : dto.getLinks()) {
                link.setId(UUID.randomUUID().toString().replace("-", ""));
                link.setInfoId(dto.getId());
                link.setCreateBy(userDTO.getId());
                link.setCreateDate(new Date());
                link.setUpdateBy(userDTO.getId());
                link.setUpdateDate(new Date());
                link.setDelFlag(0);
                projectLinkMapper.insert(link);
            }
        }
        // 修改项目管理列表成员信息(先删除原有数据，再保存)
        if (CollectionUtils.isNotEmpty(dto.getMembers())) {
            projectMemberMapper.deleteByInfoId(dto.getId());
            for (ProgramProjectListMember member : dto.getMembers()) {
                member.setId(UUID.randomUUID().toString().replace("-", ""));
                member.setInfoId(dto.getId());
                member.setCreateBy(userDTO.getId());
                member.setCreateDate(new Date());
                member.setUpdateBy(userDTO.getId());
                member.setUpdateDate(new Date());
                member.setDelFlag(0);
                projectMemberMapper.insert(member);
            }
        }
        // 修改附件信息
        if (CollectionUtils.isNotEmpty(dto.getFiles())) {
            workContractService.updateFiles(dto.getFiles(), userDTO, dto.getId());
        }
        return dto.getId();
    }

    public ProjectListDto findById(String id) {
        ProjectListDto dto = new ProjectListDto();
        // 查询主表信息
        ProgramProjectListInfo info = projectListMapper.selectById(id);
        BeanUtils.copyProperties(info, dto);
        // 查询联系人列表
        LambdaQueryWrapper<ProgramProjectListLink> linkWrapper = new LambdaQueryWrapper<>();
        linkWrapper.eq(ProgramProjectListLink::getInfoId, id);
        linkWrapper.eq(ProgramProjectListLink::getDelFlag, 0);
        List<ProgramProjectListLink> links = projectLinkMapper.selectList(linkWrapper);
        if (CollectionUtils.isNotEmpty(links)) {
            dto.setLinks(links);
        }
        // 查询成员列表
        LambdaQueryWrapper<ProgramProjectListMember> memberWrapper = new LambdaQueryWrapper<>();
        memberWrapper.eq(ProgramProjectListMember::getInfoId, id);
        memberWrapper.eq(ProgramProjectListMember::getDelFlag, 0);
        List<ProgramProjectListMember> members = projectMemberMapper.selectList(memberWrapper);
        if (CollectionUtils.isNotEmpty(members)) {
            dto.setMembers(members);
        }
        // 查询附件信息
        List<WorkAttachmentDto> files = workContractInfoMapper.findDtos(id);
        if (CollectionUtils.isNotEmpty(files)) {
            for (WorkAttachmentDto i : files) {
                i.setCreateBy(UserUtils.get(i.getBy()));
            }
            dto.setFiles(files);
        }
        return dto;
    }

    public ProgramAuditDto findByIdAudit(String id){
        ProgramAuditDto byIdAudit = programAuditMapper.findByIdAudit(id);
        // 查询附件信息
        List<WorkAttachmentDto> files = workContractInfoMapper.findDtos(id);
        if (CollectionUtils.isNotEmpty(files)) {
            for (WorkAttachmentDto i : files) {
                i.setCreateBy(UserUtils.get(i.getBy()));
            }
            byIdAudit.setWorkAttachmentDtoList(files);
        }
        return byIdAudit;
    }

    public ProgramArchiveDto findByIdArchive(String id){
        ProgramArchiveDto programArchiveDto = programArchiveMapper.findByIdArchive(id);
        if(ObjectUtil.isNotEmpty(programArchiveDto)){
            if(ObjectUtil.isNotEmpty(programArchiveDto.getProgramProjectListInfo())){
                if(StringUtils.isNotBlank(programArchiveDto.getProgramProjectListInfo().getProjectManager())){
                    UserDTO userDTO = UserUtils.get(programArchiveDto.getProgramProjectListInfo().getProjectManager());
                    programArchiveDto.getProgramProjectListInfo().setOfficeId(userDTO.getOfficeDTO().getName());
                }
            }
        }
        // 查询附件信息 - 评估报告
        List<WorkAttachmentDto> reportFileList = workContractInfoMapper.findByAttachmentIdAndFlag(id, "report");
        if (CollectionUtils.isNotEmpty(reportFileList)) {
            for (WorkAttachmentDto i : reportFileList) {
                i.setCreateBy(UserUtils.get(i.getBy()));
            }
            programArchiveDto.setReportFileList(reportFileList);
        }

        // 查询附件信息 - 评估说明
        List<WorkAttachmentDto> explainFileList = workContractInfoMapper.findByAttachmentIdAndFlag(id, "explain");
        if (CollectionUtils.isNotEmpty(explainFileList)) {
            for (WorkAttachmentDto i : explainFileList) {
                i.setCreateBy(UserUtils.get(i.getBy()));
            }
            programArchiveDto.setExplainFileList(explainFileList);
        }

        // 查询附件信息 - 评估明细表
        List<WorkAttachmentDto> detailFileList = workContractInfoMapper.findByAttachmentIdAndFlag(id, "detail");
        if (CollectionUtils.isNotEmpty(detailFileList)) {
            for (WorkAttachmentDto i : detailFileList) {
                i.setCreateBy(UserUtils.get(i.getBy()));
            }
            programArchiveDto.setDetailFileList(detailFileList);
        }

        // 查询附件信息 - 评估操作计算底稿
        List<WorkAttachmentDto> papersFileList = workContractInfoMapper.findByAttachmentIdAndFlag(id, "papers");
        if (CollectionUtils.isNotEmpty(papersFileList)) {
            for (WorkAttachmentDto i : papersFileList) {
                i.setCreateBy(UserUtils.get(i.getBy()));
            }
            programArchiveDto.setPapersFileList(papersFileList);
        }

        // 查询附件信息 - 其他
        List<WorkAttachmentDto> otherFileList = workContractInfoMapper.findByAttachmentIdAndFlag(id, "other");
        if (CollectionUtils.isNotEmpty(otherFileList)) {
            for (WorkAttachmentDto i : otherFileList) {
                i.setCreateBy(UserUtils.get(i.getBy()));
            }
            programArchiveDto.setOtherFileList(otherFileList);
        }


        return programArchiveDto;
    }

    public String deleteById(String id) {
        projectListMapper.deleteById(id);
        projectLinkMapper.deleteByInfoId(id);
        projectMemberMapper.deleteByInfoId(id);
        // 附件删除
        LambdaQueryWrapper<WorkAttachment> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(WorkAttachment::getAttachmentId, id);
        ossServiceMapper.delete(wrapper);
        return "操作成功";
    }

    public IPage<WorkContractInfo> findContractPageList(ContractDto dto, Page<WorkContractInfo> page) {
        return projectListMapper.findContractPageList(page, dto);
    }

    public String updateStatusById(ProjectListDto dto) {
        projectListMapper.updateStatusById(dto.getId(), dto.getStatus());
        return "操作成功";
    }

    public String updateStatusByAuditId(ProjectListDto dto) {
        ProgramAudit programAudit = new ProgramAudit();
        programAudit.setId(dto.getId());
        programAudit.setStatus(dto.getStatus());
        programAuditMapper.update(programAudit,new QueryWrapper<ProgramAudit>().lambda().eq(ProgramAudit::getId,programAudit.getId()));
        return "操作成功";
    }

    public String updateStatusByArchiveId(ProjectListDto dto) {
        ProgramArchive programArchive = new ProgramArchive();
        programArchive.setId(dto.getId());
        programArchive.setStatus(dto.getStatus());
        programArchiveMapper.update(programArchive,new QueryWrapper<ProgramArchive>().lambda().eq(ProgramArchive::getId,programArchive.getId()));
        return "操作成功";
    }

    public IPage<ProgramProjectListInfo> list(Page<ProgramProjectListInfo> page, ProgramProjectListInfo info) throws Exception{
        QueryWrapper<ProgramProjectListInfo> wrapper = QueryWrapperGenerator.buildQueryCondition(info, ProgramProjectListInfo.class);
        wrapper.eq("a.del_flag", "0");
        if (info.getCreateDates() != null) {
            wrapper.between("a.create_date", info.getCreateDates()[0], info.getCreateDates()[1]);
        }
        IPage<ProgramProjectListInfo> pageList = projectListMapper.findPageList(page, wrapper);
        pageList.getRecords().stream().forEach(item ->{
            String isInvoice = projectListMapper.selectInvoiceFlag(item.getId());
            item.setIsInvoice(isInvoice);
        });
        return pageList;
    }

    public List<ExportFileDto> exportList(ProgramProjectListInfo info) throws Exception{
        QueryWrapper<ProgramProjectListInfo> wrapper = QueryWrapperGenerator.buildQueryCondition(info, ProgramProjectListInfo.class);
        wrapper.eq("a.del_flag", "0");
        if (info.getCreateDates() != null) {
            wrapper.between("a.create_date", info.getCreateDates()[0], info.getCreateDates()[1]);
        }
        List<ExportFileDto> list = projectListMapper.findList(wrapper);
        // 状态值处理
        if (CollectionUtils.isNotEmpty(list)) {
            for(ExportFileDto i : list) {
                if ("0".equals(i.getStatus())) {
                    i.setStatus("未登记");
                }
                if ("1".equals(i.getStatus())) {
                    i.setStatus("暂存");
                }
                if ("2".equals(i.getStatus())) {
                    i.setStatus("登记中");
                }
                if ("3".equals(i.getStatus())) {
                    i.setStatus("撤回");
                }
                if ("4".equals(i.getStatus())) {
                    i.setStatus("驳回");
                }
                if ("5".equals(i.getStatus())) {
                    i.setStatus("登记完成");
                }
            }
        }
        return list;
    }

    public String saveReportNo(String id) throws Exception{
        UserDTO userDTO = UserUtils.getCurrentUserDTO();
        ProgramProjectListInfo programProjectListInfo = projectListMapper.selectById(id);
        String reportNo = "";
        if(ObjectUtil.isNotEmpty(programProjectListInfo)){
            if(StringUtils.isNotBlank(programProjectListInfo.getReportType())){
                if (programProjectListInfo.getReportType().equals("1")){
                    reportNo = serialnumTplService.genSerialNum(userDTO.getCompanyDTO().getId(), ProgramReportNoDto.BIZ_CODE);
                } else if (programProjectListInfo.getReportType().equals("2")) {
                    reportNo = serialnumTplService.genSerialNum(userDTO.getCompanyDTO().getId(), ProgramReportNoDto.BIZ_CODE_);
                }
                ProgramReportNo programReportNo = new ProgramReportNo();
                programReportNo.setProgramId(id);
                programReportNo.setReportNo(reportNo);
                programReportNoMapper.insert(programReportNo);
            }
        }
        return reportNo;
    }
}
